#ifndef __DLLPLUGIN_H__
#define __DLLPLUGIN_H__

#include <vector>
#include "kakuto3.h"
#include "PluginInterface.h"

class DllPlugin : public PluginInterface{
public:
	DllPlugin() : m_dll(NULL),m_PluginUnInit(NULL){}
	~DllPlugin(){if(m_dll)::FreeLibrary(m_dll);}

	bool Load(const TCHAR *file);
	bool UnLoad(){BOOL r = ::FreeLibrary(m_dll);m_dll = NULL;return r ? true : false;}
	const TCHAR *PluginFile(){return m_dllname.c_str();}
	const TCHAR *Info(int index){return m_infos[index].info.c_str();}
	const TCHAR *SoftName(int index){return m_infos[index].softname.c_str();}
	void Set(HWND hwnd,int index){(*m_Set)(hwnd,ablib::TransCode::ToChar(m_infos[index].softname).c_str());}
	bool GetInfo(HWND hwnd){return (*m_GetInfo)(hwnd);}
	bool IsDefault(int index){return (*m_IsDefault)(ablib::TransCode::ToChar(m_infos[index].softname).c_str());}
	PluginInterface::DownloadData GetDownloadData(int index);
	bool Install(HWND hwnd,int index,const std::map<ablib::string,State> &downloadstates);
	bool Sets(int index){return (*m_Sets)(ablib::TransCode::ToChar(m_infos[index].softname).c_str());}
	bool Reboots(){return (*m_Reboots)();}
	void UnInit(){if(m_PluginUnInit)(*m_PluginUnInit)();}

	int GetSoftNum(){return m_infos.size();}

private:
	ablib::string m_dllname;
	HMODULE m_dll;

	typedef ::PluginInfo* (*INFO)();
	typedef bool (*PLUGININIT)(const char*,const char*);
	typedef void (*SET)(HWND,const char*);
	typedef bool (*GETINFO)(HWND);
	typedef bool (*ISDEFAULT)(const char*);
	typedef ::DownloadData (*GETDOWNLOADDATA)(const char*);
	typedef bool (*INSTALL)(HWND,const char*,::DLResult);
	typedef bool (*REBOOTS)();
	typedef bool (*SETS)(const char*);
	typedef void (*PLUGINUNINIT)();

	INFO m_Info;
	SET m_Set;
	PLUGININIT m_PluginInit;
	GETINFO m_GetInfo;
	ISDEFAULT m_IsDefault;
	GETDOWNLOADDATA m_GetDownloadData;
	INSTALL m_Install;
	REBOOTS m_Reboots;
	SETS m_Sets;
	PLUGINUNINIT m_PluginUnInit;

	struct SoftInfo{
		SoftInfo(const ablib::string &s,const ablib::string &i) : softname(s),info(i){}
		ablib::string softname,info;
	};
	std::vector<SoftInfo> m_infos;
};

#endif //__DLLPLUGIN_H__

