#include "stdafx.h"
#include "encode.h"

ablib::basic_string<char> tmp_cstring;


namespace{
	enum CodeSet{ ASCII, KANA, JIS};

	//JIS ZX 0208  Shift_JIS
	void jis2sjis(BYTE k,BYTE t,char *rv){
		if(k>=0x3f)	rv[0] = (char)(((k+1) >> 1) + 0xc0);
		else		rv[0] = (char)(((k+1) >> 1) + 0x80);
		if( k&1 )	rv[1] = (char)((t >> 6) ? t + 0x40 : t + 0x3f);
		else		rv[1] = (char)(t + 0x9e);
	}

	inline int tosjis(CodeSet cs,char *rv,const BYTE *b,const BYTE *end){
		BYTE c[2];
		switch(cs){
		case ASCII:
			if(b >= end)return 0;
			rv[0] = *b;
			return 1;
		case JIS:
			if(b >= end - 1)return 0;
			c[0] = *b;c[1] = *(b + 1);
			jis2sjis((c[0] & 0x7F) - 0x20,(c[1] & 0x7F) - 0x20,rv);
			return 2;
		case KANA:
			if(b >= end)return 0;
			rv[0] = *b;
			return 1;
		default:
			return 0;
		}
	}
	inline BYTE GetData(const BYTE *i,const BYTE *end){return (i < end ? *i : 0);}
}

const char *Encode::EUC2SJIS(const BYTE *b,size_type size){
	const BYTE SS2 = 0x8E;
	const BYTE SS3 = 0x8F;

	char tmp[4];

	tmp_cstring = "";
	tmp_cstring.reserve(size);
	CodeSet G[4] = {ASCII,JIS,KANA,JIS};
	const BYTE *end = b + size;
	while(b < end){
		if(*b == '\r' || *b == '\n'){
			tmp_cstring += '\n';
			if(*(b++) == '\r' && GetData(b,end) == '\n')++b;
		}else{
			switch(*b){
			case SS2:
				++b;
				::ZeroMemory(tmp,sizeof(char)*4);
				b += tosjis(G[2],tmp,b,end);
				tmp_cstring += tmp;
				break;
			case SS3:
				++b;
				::ZeroMemory(tmp,sizeof(char)*4);
				b += tosjis(G[3],tmp,b,end);
				tmp_cstring += tmp;
				break;
			default:
				::ZeroMemory(tmp,sizeof(char)*4);
				if(*b >= 0x80){
					b += tosjis(G[1],tmp,b,end);
					tmp_cstring += tmp;
				}else{
					b += tosjis(G[0],tmp,b,end);
					tmp_cstring += tmp;
				}
			}
		}
	}
	return tmp_cstring.c_str();
}

const char *Encode::JIS2SJIS(const BYTE *b,size_type size){
	static const int ESC = 0x1B;

	char tmp[4];

	tmp_cstring = "";
	tmp_cstring.reserve(size);
	const BYTE *end = b + size;
	CodeSet G0 = ASCII;
	while(b < end){
		if(*b == '\r' || *b == '\n'){
			G0 = ASCII;
			tmp_cstring += '\n';
			if(*(b++) == '\r' && GetData(b,end) == '\n')++b;
		}else if(*b == ESC){
			++b;
			if(*b == '('){
				++b;
				if(*b == 'B' || *b == 'J'){++b;G0 = ASCII;}
			}else if(*b == '$'){
				++b;
				if(*b == '@' || *b == 'B'){++b;G0 = JIS;}
			}
		}else{
			::ZeroMemory(tmp,sizeof(char)*4);
			b += tosjis(G0,tmp,b,end);
			tmp_cstring += tmp;
		}
	}
	return tmp_cstring.c_str();
}


