# -*- encoding: Shift_JIS -*-

require 'net/http'
require 'uri'
require 'time'
require 'nkf'
require 'tmpdir'
require 'pathname'

Net::HTTP.version_1_2

class BeamerClass
	def initialize
		@ini = ""
		@log = ""
		
		@url = {
			"beamer" => "http://www.dnsbalance.ring.gr.jp/pub/text/CTAN/macros/latex/contrib/beamer.zip",
			"xcolor" => "http://www.dnsbalance.ring.gr.jp/pub/text/CTAN/macros/latex/contrib/xcolor.zip",
			"pgf" => "http://www.dnsbalance.ring.gr.jp/pub/text/CTAN/graphics/pgf/base.zip"
		}
		@localfile = {
			"beamer" => "beamer.zip",
			"xcolor" => "xcolor.zip",
			"pgf" => "pgf.zip"
		}
		@definstall = {"beamer" => true,"xcolor" => true,"pgf" => true}
		@size = {};
		@success = {};
	end
	
	def info
		return [
			{"softname" => "beamer", "info" => "プレゼンテーション用クラスファイル"},
			{"softname" => "xcolor", "info" => "beamerに必要"},
			{"softname" => "pgf",    "info" => "beamerに必要"}
		]
	end
	
	
	def plugin_init(ini,log)
		@ini = ini
		@log = log
	end
	
	
	def reboot?()
		return false
	end
	
	
	def set?(soft)
		return false
	end
	
	
	def set(hwnd,soft)
		return
	end
	
	
	def get_info_by_soft(hwnd,soft)
		Log("#{soft}の情報取得開始");
		
		if AbTeXinst::Setting::use_network? == false then
			begin
			@definstall[soft] = File.exist?(AbTeXinst::Setting::hozon_dir + @localfile[soft]);
			rescue
				@definstall[soft] = false;
			end
			return true;
		end
		uri = URI.parse(@url[soft])
		if uri.scheme != "http" then
			Log("httpでないといけません．")
			return false
		end
		AbTeXinst::add_message(hwnd,"\r\n\r\n♪#{soft}の情報取得開始♪\r\n")
		response = nil
		
		proxy = AbTeXinst::Setting::proxy;
		if proxy == nil then
			proxy = AbTeXinst::get_IEproxy_info;
		end
		if proxy == nil or proxy["server"] == "" then
			proxy = {"server" => nil,"port" => nil};
		end
		AbTeXinst::add_message(hwnd,"♪" + uri.host + "に接続し情報を取得します．\r\n");
		Net::HTTP::Proxy(proxy["server"],proxy["port"]).start(uri.host){|http|
			response = http.head(uri.path)
		}
		
		
		if response == nil then
			AbTeXinst::add_message(hwnd,"情報取得に失敗\r\n")
			Log("情報取得に失敗")
			return false
		end
		
		
		@size[soft] = response['Content-Length'].to_i
		nettime = Time.parse(response['last-modified']) + 120
		begin
			localtime = File.atime(AbTeXinst::Setting::hozon_dir + @localfile[soft]);
			@definstall[soft] = (localtime < nettime)
		rescue
			@definstall[soft] = true
		end
		
		Log("#{soft}の情報取得終了")
		AbTeXinst::add_message(hwnd,"完了\r\n")
		@success[soft] = true
		return true
	end
	
	def get_info(hwnd)
		r = get_info_by_soft(hwnd,"beamer");
		r = get_info_by_soft(hwnd,"xcolor") and r;
		r = get_info_by_soft(hwnd,"pgf") and r;
		return r;
	end
	
	
	def download_data(soft)
		if AbTeXinst::Setting::use_network? == 0 then
			return {
				"success" => true,
				"download_files" => []};
		else
			return {
				"success" => @success[soft],
				"download_files" => [{
					"url" => @url[soft],
					"file" => @localfile[soft],
					"size" => @size[soft]}]}
		end
	end
	
	
	def default?(soft)
		return @definstall[soft]
	end
	
	
	def install(hwnd,soft,dlresult)
		Log("#{soft}のインストール");
		if AbTeXinst::Setting::use_network? == true then
			if dlresult[@url[soft]] == nil or dlresult[@url[soft]] != AbTeXinst::SUCCESS then
				return false
			end
		end
		
		dir = AbTeXinst::Setting::install_dir + "share\\texmf-local\\tex\\latex\\" + soft;
		from = AbTeXinst::Setting::hozon_dir + "\\" + @localfile[soft];
		if AbTeXinst::zipdlg(hwnd,from,dir) == true then
			AbTeXinst::add_message(hwnd,"成功\r\n")
			Log("#{soft}のインストールに成功")
			return true
		else
			AbTeXinst::add_message(hwnd,"失敗\r\n")
			Log("#{soft}のインストールに失敗")
			return false
		end
	end
	
private
	def Log(l)
		begin
			fp = File.open(@log,"a+")
		rescue
			return
		end
		fp.write(l + "\n");
		fp.close()
	end
	
end


Beamer = BeamerClass.new

