#ifndef __GETINFO_H__
#define __GETINFO_H__

#include "FtpAccess.h"
#include "HttpAccess.h"
#include <system/Time.h>
#include "../kakuto3.h"
#include <vector>

class GetInfo{
public:
	GetInfo(){}
	~GetInfo(){}
	bool Get(HWND parent);
	const TCHAR *GetURL() const{return m_url.c_str();}
	bool IsSuccess() const{return m_issuccess;}
	size_type GetSize() const{return m_size;}
	bool IsDefaultInstall() const{return m_defaultinstall;}

private:
	struct FileData{
		ablib::string file;
		int size;
		ablib::system::Time time;
	};

	struct network{
		virtual bool Connect(int timeout = 20) = 0;
		virtual bool SetCurDir(const TCHAR *dir) = 0;
		virtual bool GetFileList(std::vector<GetInfo::FileData> &rv) = 0;
		virtual ablib::string GetProtocol() = 0;
	};

	ablib::string m_url,m_server,m_curdir;
	size_type m_size;
	HWND m_parent;
	bool m_issuccess,m_defaultinstall;
	bool GetFromNetwork(network *net,HWND parent);
	bool GetFromLocal(HWND parent);

	void AddMsg(const ablib::string &msg){::SendMessage(m_parent,WM_ADDMESSAGE,0,reinterpret_cast<LPARAM>(msg.c_str()));}
	bool CheckCancel(){return ::SendMessage(m_parent,WM_ISCANCEL,0,0) != FALSE;}
	bool Cancel();
	bool Error(const ablib::string &log);
	FileData GetLatest(const std::vector<FileData> &files);


	struct FTP : public network{
		FTP(const TCHAR *server,const TCHAR *proxy) : m_ftp(server,_T("anonymous"),_T("a@b")){}
		~FTP();
		bool Connect(int timeout){return m_ftp.Connect(timeout);}
		bool GetFileList(std::vector<GetInfo::FileData> &rv);
		bool SetCurDir(const TCHAR *dir){return m_ftp.cd(dir);}
		bool SetProxy(const TCHAR *proxy){return false;}
		ablib::string GetProtocol(){return _T("ftp");}
	private:
		FtpAccess m_ftp;
	};

	struct HTTP : public network{
		GetInfo::HTTP::HTTP(const TCHAR *server,const TCHAR *proxy) : m_http(server,proxy),m_curdir(_T("/")){}
		~HTTP(){}
		bool Connect(int timeout){return m_http.Connect(timeout);}
		bool SetCurDir(const TCHAR *dir){m_curdir = dir;return true;}
		bool GetFileList(std::vector<GetInfo::FileData> &rv);
		ablib::string GetProtocol(){return _T("http");}
	private:		
		HttpAccess m_http;
		ablib::string m_curdir;

		bool GetFileListFromIndexHtml(std::vector<GetInfo::FileData> &fd,const ablib::string &str);
		DWORD FromStrMonth2IntMonth(const ablib::string &s);
		int FromStrSize2IntSize(const ablib::string &s);
	};


};


#endif //__GETINFO_H__

